package WoDo::BASE;

use JSON;
use LWP::Simple;
use RDF::Redland;
use URI;

sub _load_from_uri
{
	my (undef, $uri) = @_;
	my $self = {};
	
	my $data = from_json(get($uri));

	my $URI = URI->new($uri);
	$data->{'class'}     = ''.URI->new_abs($data->{'class'}, $URI);
	$data->{'graph_src'} = ''.URI->new_abs($data->{'graph_src'}, $URI);
	
	my $storage = new RDF::Redland::Storage("hashes", "test", "new='yes',hash-type='memory'");
	my $model   = new RDF::Redland::Model($storage, "");
	my $parser  = new RDF::Redland::Parser("guess");
	my $ruri    = new RDF::Redland::URI($data->{'graph_src'});
	$parser->parse_into_model($ruri, $ruri, $model);
	
	$class = WoDo::FACTORY->load_class($data->{'class'});

	$self->{webid} = $data->{webid};
	$self->{graph} = $model;
	
	bless $self, $class;
}

sub _sparql_first
{
	my ($self, $querystring) = (shift, shift);
	$querystring = $self->_substitute($querystring, @_);

	my $query = RDF::Redland::Query->new($querystring,
		RDF::Redland::URI->new($self->{webid}),
		undef,
		'sparql');
	
	my $results = $self->{graph}->query_execute($query);
	RESULT: while (! $results->finished)
	{
		my $node = $results->binding_value(0);
		if ($node->type == $RDF::Redland::Node::Type_Resource)
			{ return $node->uri->as_string; }
		elsif ($node->type == $RDF::Redland::Node::Type_Literal)
			{ return $node->literal_value; }
		
		$results->next_result;
	}
	
	return undef;
}

sub _sparql_array
{
	my ($self, $querystring) = (shift, shift);
	$querystring = $self->_substitute($querystring, @_);

	my $query = RDF::Redland::Query->new($querystring,
		RDF::Redland::URI->new($self->{webid}),
		undef,
		'sparql');
	
	my $rv = [];
	
	my $results = $self->{graph}->query_execute($query);
	RESULT: while (! $results->finished)
	{
		my $node = $results->binding_value(0);
		if ($node->type == $RDF::Redland::Node::Type_Resource)
			{ push @$rv, $node->uri->as_string; }
		elsif ($node->type == $RDF::Redland::Node::Type_Literal)
			{ push @$rv, $node->literal_value; }
		
		$results->next_result;
	}
	
	return $rv;
}

sub _sparql_objects
{
	my ($self, $querystring) = (shift, shift);
	$querystring = $self->_substitute($querystring, @_);

	my $query = RDF::Redland::Query->new($querystring,
		RDF::Redland::URI->new($self->{webid}),
		undef,
		'sparql');
	
	my $rv = [];
	
	my $results = $self->{graph}->query_execute($query);
	RESULT: while (! $results->finished)
	{
		my $row = {};
		for (my $i=0; $i < $results->bindings_count(); $i++)
		{
			my $name = $results->binding_name($i);
			my $node = $results->binding_value($i);
			
			if ($node->type == $RDF::Redland::Node::Type_Resource)
				{ $row->{$name} = $node->uri->as_string; }
			elsif ($node->type == $RDF::Redland::Node::Type_Literal)
				{ $row->{$name} = $node->literal_value; }
		}
		push @$rv, $row;
	}
	
	return $rv;
}

sub _substitute
{
	my $self   = shift;
	my $string = $_[0];
	
	$string =~ s/!!0/$self->{webid}/;
	
	for (my $i=1; $i < scalar(@_); $i++)
	{
		$string =~ s/!!$i/$_[$i]/ge;
	}
	
	return $string;
}

1;
