package WoDo::FACTORY;

use Digest::SHA1 qw(sha1_hex);
use JSON qw(from_json);
use LWP::Simple qw(get);

sub load_class
{
	my ($class, $klass) = @_;
	my ($url, $klass) = split /#/, $klass;
	WoDo::FACTORY->load_url($url);
	return 'WoDo::' . sha1_hex($url) . '::' . $klass;
}

sub load_url
{
	my ($class, $uri) = @_;
	my $data = get($uri);
	my $classes = from_json($data);
	
	foreach my $class (keys %$classes)
	{
		WoDo::FACTORY->define_package('WoDo::'.sha1_hex($uri).'::'.$class,
			$classes->{$class});
	}
}

sub define_package
{
	my ($class, $target_class, $methods) = @_;
		
	my $klass;
	$klass .= sprintf("package \%s;\n", $target_class);
	$klass .= sprintf("\@ISA = qw(WoDo::BASE);\n");
	
	foreach my $method_name (keys %$methods)
	{
		my $sparql = $methods->{$method_name}->{'sparql'};
		my $return = $methods->{$method_name}->{'return'} || 'objects';
		
		die "Undefined return type: \"return\".\n"
			unless ($return =~ /^(first|array|objects)$/)
			;
		die "Diallowed method name: \"$method_name\".\n"
			if $method_name =~ /[^A-Za-z0-9_]/;
		$sparql =~ s/\'/\\\'/g;
		
		$klass .= "sub $method_name { my (\$self) = shift; return \$self->_sparql_$return(\'$sparql\', \@_); }\n";
	}
	
	$klass .= "1;\n";

	eval($klass);
	die $@."\n" if $@;
}

1;
